function [r d sigmaU] = BuildBDTtree_yields_vols(rate,sigma,Pu,Pd,N,dt,ObjFun)

% Black-Derman-Toy interest rate fitted to yields and volatilities.
% Required inputs
% rate   = Initial interest rate (scalar).
% sigma  = Initial volatility (scalar).
% Pu     = Pu vector (size N+1).
% Pd     = Pd vector (size N+1).
% N      = number of time steps.
% ObjFun = 'SSE' or 'ABS': Objective function to find U and sigma.

% Initialize the matrices
d  = zeros(2*N+1,N+1);
r  = zeros(2*N+1,N+1);
Qu = zeros(2*N+1,N+1);
Qd = zeros(2*N+1,N+1);

% Fill in starting values for the matrices
r(N+1,1)  = rate;
d(N+1,1)  = 1/(1+r(N+1,1)*dt);
Qu(N,2)   = 1;
Qd(N+2,2) = 1;
U(1)      = rate;
sigmaU(1) = sigma;

% Create rate step indices.
J = [N:-1:-N];

% Settings for the optimization.
options = optimset('TolFun', 1e-20);
start = [.05 .1];

% Construct the BDT tree from node 2 onwards.
for j=2:N+1;
	Iu = [N-j+2:2:N+j-2];
	Id = [N-j+4:2:N+j];
	% Find U(j) and sigma(j)
	param = fsolve(@(U) findU_and_sigma(U,Qu(Iu,j),Qd(Id,j),dt,Pu(j),Pd(j),J(Iu),J(Id),ObjFun), start, options);
	U(j)      = param(1);
	sigmaU(j) = param(2);
	I = [N-j+2:2:N+j];
	% Fit the interest rates and discount factors
	r(I,j) = U(j).*exp(sigmaU(j).*J(I).*dt);
	d(I,j) = 1./(1+r(I,j).*dt);
	% Update Qu and Qd
	if j<=N
		for i=N-j+1:N+j-1
			if i==N-j+1
				Qu(i,j+1) = 0.5*Qu(i+1,j)*d(i+1,j);
			elseif i==N+j-1
				Qu(i,j+1) = 0.5*Qu(i-1,j)*d(i-1,j);
			else
				Qu(i,j+1) = 0.5*Qu(i+1,j)*d(i+1,j) + 0.5*Qu(i-1,j)*d(i-1,j);
			end
		end
		for i=N-j+3:N+j+1
			if i==N-j+3
				Qd(i,j+1) = 0.5*Qd(i+1,j)*d(i+1,j);
			elseif i==N+j+1
				Qd(i,j+1) = 0.5*Qd(i-1,j)*d(i-1,j);
			else
				Qd(i,j+1) = 0.5*Qd(i+1,j)*d(i+1,j) + 0.5*Qd(i-1,j)*d(i-1,j);
			end
		end
	end
end

